# -*- encoding: utf-8 -*-

import difflib
import os
import tempfile

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import mock
from six import StringIO

from .helpers import get_compose_path, get_fixture

from compose_utils import repo_file


class TestRepoFile(unittest.TestCase):
    def setUp(self):
        _, self.temp_file = tempfile.mkstemp(prefix="repo-file-")

    def tearDown(self):
        os.remove(self.temp_file)

    def assertFilesEqual(self, fn1, fn2, strip=None):
        with open(fn1, "r") as f1:
            lines1 = f1.readlines()
        with open(fn2, "r") as f2:
            lines2 = f2.readlines()
            if strip:
                lines2 = [line.replace(strip, "") for line in lines2]
        diff = "".join(
            difflib.unified_diff(lines1, lines2, fromfile="EXPECTED", tofile="ACTUAL")
        )
        self.assertEqual(diff, "", "Files differ:\n" + diff)

    def success_run(self, args, compose_id, expected):
        repo_file.main(
            args + ["--output", self.temp_file, get_compose_path(compose_id)]
        )
        strip = os.path.dirname(__file__) if "--translate" not in args else None
        self.assertFilesEqual(get_fixture(expected), self.temp_file, strip=strip)

    def test_no_filter(self):
        self.success_run([], "DP-1.0-20160315.t.0", "default.repo")

    def test_translate(self):
        self.success_run(
            ["--translate", os.path.dirname(__file__) + ",http://example.com"],
            "DP-1.0-20160315.t.0",
            "translate.repo",
        )

    def test_with_source(self):
        self.success_run(["--include-source"], "DP-1.0-20160315.t.0", "sources.repo")

    def test_with_debuginfo(self):
        self.success_run(
            ["--include-debuginfo"], "DP-1.0-20160315.t.0", "debuginfo.repo"
        )

    def test_skip_variant(self):
        self.success_run(["--variant=Client"], "DP-1.0-20160315.t.0", "client.repo")

    def test_hardcode_arch(self):
        self.success_run(["--arch=x86_64"], "DP-1.0-20160315.t.0", "x86_64.repo")

    def test_no_paths(self):
        self.success_run([], "DP-1.0-20181012.t.0", "empty.repo")

    def test_bad_hardcode_arch(self):
        with mock.patch("sys.stderr", new_callable=StringIO) as mock_err:
            with self.assertRaises(SystemExit):
                repo_file.main(
                    [
                        "--output",
                        self.temp_file,
                        "--arch=ppc64le",
                        get_compose_path("DP-1.0-20160315.t.0"),
                    ]
                )
        self.assertIn(
            "Variant Client does not have arch ppc64le.\n", mock_err.getvalue()
        )

    def test_bad_translate(self):
        with mock.patch("sys.stderr", new_callable=StringIO) as mock_err:
            with self.assertRaises(SystemExit):
                repo_file.main(
                    [
                        "--output",
                        self.temp_file,
                        get_compose_path("DP-1.0-20160315.t.0"),
                        "--translate",
                        "/foo/bar,http://example.com",
                    ]
                )
        self.assertIn(
            "/Client/$basearch/os does not start with /foo/bar.\n", mock_err.getvalue()
        )
